<TeXmacs|1.0.4.5>

<style|mmxdoc>

<\body>
  <tmdoc-title|Symbolic expressions>

  <section|Introduction>

  The aim of the <verbatim|symbolic> class is to provide a uniform interface
  for a wide variety of <em|syntactic> symbolic expression types. The
  symbolic class is really a ``contextual template type''
  <verbatim|symbolic\<less\>C\<gtr\>>, with<nbsp><verbatim|C> of type
  <verbatim|context>. This means that one should think of the class as
  carrying a template parameter<nbsp><verbatim|C>, but this parameter is
  really a global variable which may be changed during computations.

  The <verbatim|context> class determines a ``sufficient simple''
  mathematical theory, so that free expressions in the theory can be
  represented in a canonical and syntactic way. For instance, if the context
  is ``free commutative algebra'', then concrete symbolic objects might be
  polynomials. In general, the context rather specifies properties for
  objects, so that certain variables behave like commutative variables, other
  variables are systematically expanded, and yet other variables do not
  satisfy any relations.

  From the internal point of view, symbolic objects are represented by unique
  pointers. In other words, whenever a symbolic expression is constructed, we
  first test whether this symbolic expression already exists in memory and
  use a pointer copy of the expression in memory if this is the
  case<nbsp>(recall: we might retain the <em|simplest> representation). This
  strategy is possible, because we assumed that terms in the theory from the
  context admit canonical expressions. Expressions are compared
  (<verbatim|==>, <samp|<verbatim|!=>>, <verbatim|\<less\>=>,
  <verbatim|\<less\>>, <verbatim|\<gtr\>=>, <verbatim|\<gtr\>>) and hashed
  using their pointers. In the case of <verbatim|\<less\>=>,
  <verbatim|\<less\>>, <verbatim|\<gtr\>=>, <verbatim|\<gtr\>>, this should
  not be mixed up with possible other significations of these relation (in
  which a new class should be constructed on top of <verbatim|symbolic>).

  Typically, the <verbatim|symbolic> class should be thought of as a union of
  classes of the following type

  <\itemize>
    <item>One class for atomic symbols like <with|mode|math|x>,
    <with|mode|math|y>, <abbr|etc.>

    <item>Atomic classes, like integers, rational numbers, etc.

    <item>One class for syntactic compound objects like <with|mode|math|f(x)>
    or <with|mode|math|f(2)>.

    <item>Additional classes for the efficient representation of special
    types of compound objects, like commutative sums, associative products,
    <abbr|etc.>
  </itemize>

  In addition, the symbolic class might carry out a limited amount of term
  rewriting. This presupposes the existence of pattern matching for the
  symbolic type (<abbr|i.e.> for all classes in the union) and the fact that
  the term rewriting ends up with canonical objects (<abbr|i.e.> if the
  Church-Rosser property is satisfied).

  In general, the <verbatim|symbolic> class is <em|not> meant to encode
  objects in complicated models. Instead, the <verbatim|symbolic> class is
  used to provide a suitable syntactical objects (better than reasoning on
  mere trees, since atomic classes and special types of compound classes are
  treated in a more optimal way) to <em|express> and <em|solve> problems in
  complex theories. Of course, whenever, after additional computations, a
  complex theory may be reformulated such that objects again admit canonical
  representations (<abbr|i.e.> reduced Grbner basis and reduced
  polynomials), then canonical objects in the reformulated theory may again
  be reused to recursively build more complicated symbolic classes.

  For experimental purposes, one may cheat with respect to what has been said
  above, and construct symbolic classes which correspond to models in which
  canonical representations are not beforehand. In that case, useful
  computations may still be carried out, but one has to bear in mind that the
  equality test <verbatim|==> in such models is no longer correct. In this
  case, computations in <verbatim|symbolic> only <em|approximate>
  computations in the model one wishes to represent.

  <\warning>
    The method <verbatim|identical> of <verbatim|symbolic_rep> should be mere
    syntactic comparison. Subtle errors (hard to debug) may occur when this
    method triggers the creation or destruction of symbolic objects (mere
    read-only traversal is OK), because the method is precisely called during
    the creation of symbolic objects (in order to represent the same
    expressions by a unique pointer).
  </warning>

  <section|Commutative expressions>

  The most important context is the context of commutative expressions. For
  maximal flexibility, this context is constructed on top of an arbitrary
  other commutative context ``<verbatim|scalar>''. The context of commutative
  expressions implements the operations <with|mode|math|<op|+>>,
  <with|mode|math|<op|->>, <with|mode|math|<op|\<times\>>>,
  <with|mode|math|<op|/>> and powers (besides the universal operation of
  function application and the construction of atomic expressions). Other
  symbols are either free variables in the new context or inherited from the
  scalar context.

  Commutative expressions are represented as follows:

  <\eqnarray*>
    <tformat|<table|<row|<cell|\<cal-E\>>|<cell|<with|color|blue|\<assign\>>>|<cell|0<with|color|blue|\<amalg\>>\<cal-P\><with|color|blue|\<amalg\>><left|(><big|sum>\<cal-S\><rsub|0>*\<cal-P\><with|color|blue|\<setminus\>>(0<with|color|blue|\<amalg\>>1\<times\>\<cal-T\><with|color|blue|\<amalg\>>\<cal-S\>\<times\>1)<right|)>>>|<row|<cell|\<cal-P\>>|<cell|<with|color|blue|\<assign\>>>|<cell|1<with|color|blue|\<amalg\>>\<cal-T\><with|color|blue|\<amalg\>><left|(><big|prod>\<cal-E\><rsub|0,1><rsup|\<cal-E\><rsub|0>><with|color|blue|\<setminus\>>(1<with|color|blue|\<amalg\>>\<cal-T\><rsup|1>)<right|)>>>|<row|<cell|\<cal-T\>>|<cell|<with|color|blue|\<assign\>>>|<cell|\<cal-S\><rsub|0,1><with|color|blue|\<amalg\>>\<cal-V\><with|color|blue|\<amalg\>>\<cal-E\><rsub|+,-\<times\>,/,<wide||^>>(\<cal-E\><rsup|\<ast\>>)>>>>
  </eqnarray*>

  Here

  <\itemize>
    <item><with|mode|math|\<cal-E\>> denotes the set of all expressions and
    <with|mode|math|\<cal-E\><rsup|\<ast\>>> the set of tuples of
    expressions.

    <item><with|mode|math|\<cal-P\>> denotes the set of power products.

    <item><with|mode|math|\<cal-T\>> denotes the set of ``terms''.

    <item><with|mode|math|\<cal-S\>> denotes the commutative context of
    scalars (from the base context <verbatim|scalar>).

    <item><with|mode|math|\<cal-V\>> denotes a set of ground atoms for the
    context. We assume that all expressions in <with|mode|math|\<cal-S\>> are
    free from the variables in <with|mode|math|\<cal-V\>>.

    <item><with|mode|math|\<cal-S\><rsub|0><with|color|blue|\<assign\>>\<cal-S\><with|color|blue|\<setminus\>>0>,
    <with|mode|math|\<cal-S\><rsub|0,1><with|color|blue|\<assign\>>\<cal-S\><with|color|blue|\<setminus\>>(0<with|color|blue|\<amalg\>>1)>,
    <with|mode|math|\<cal-E\><rsub|0><with|color|blue|\<assign\>>\<cal-E\><with|color|blue|\<setminus\>>0>,
    <with|mode|math|\<cal-E\><rsub|0,1><with|color|blue|\<assign\>>\<cal-E\><with|color|blue|\<setminus\>>(0<with|color|blue|\<amalg\>>1)>
    and <with|mode|math|\<cal-E\><with|color|blue|\<assign\>>\<cal-E\><with|color|blue|\<setminus\>>(<op|+><with|color|blue|\<amalg\>><op|-><with|color|blue|\<amalg\>><op|\<times\>><with|color|blue|\<amalg\>><op|/><with|color|blue|\<amalg\>><wide||^>)>.
  </itemize>

  Of course, <with|mode|math|<big|sum>\<cal-X\>> and
  <with|mode|math|<big|prod>\<cal-X\>> stand for commutative sums
  <abbr|resp.> products over <with|mode|math|\<cal-X\>>. Moreover, terms are
  assumed to be ``regrouped''; for instance, if <with|mode|math|a\<times\>x>
  and <with|mode|math|b\<times\>y> occur in
  <with|mode|math|<big|sum>\<cal-X\>>, then we must have
  <with|mode|math|x\<neq\>y>.

  Besides the axioms satisfied by commutative context of the scalars in
  <with|mode|math|\<cal-S\>>, commutative expressions always satisfy the
  following axioms:

  <\eqnarray*>
    <tformat|<table|<row|<cell|0+a>|<cell|=>|<cell|a<htab|5mm>(a\<in\>\<cal-E\>)>>|<row|<cell|(a+b)+c>|<cell|=>|<cell|a+(b+c)<htab|5mm>(a,b,c\<in\>\<cal-E\>)>>|<row|<cell|a+b>|<cell|=>|<cell|b+a<htab|5mm>(a,b\<in\>\<cal-E\>)>>|<row|<cell|a-b>|<cell|=>|<cell|a+(\<um\>1)\<times\>b<htab|5mm>(a,b\<in\>\<cal-E\>)>>|<row|<cell|0\<times\>a>|<cell|=>|<cell|0<htab|5mm>(a\<in\>\<cal-E\>)>>|<row|<cell|1\<times\>a>|<cell|=>|<cell|a<htab|5mm>(a\<in\>\<cal-E\>)>>|<row|<cell|(a+b)\<times\>c>|<cell|=>|<cell|a\<times\>c+b\<times\>c<htab|5mm>(a,b\<in\>\<cal-S\>,c\<in\>\<cal-E\>)>>|<row|<cell|a\<times\>(b\<times\>c)>|<cell|=>|<cell|(a\<times\>b)\<times\>c<htab|5mm>(a\<in\>\<cal-E\>,b\<in\>\<cal-E\>,c\<in\>\<cal-E\>)>>|<row|<cell|a\<times\>b>|<cell|=>|<cell|b\<times\>a<htab|5mm>(a\<in\>\<cal-E\>,b\<in\>\<cal-E\>)>>|<row|<cell|a/b>|<cell|=>|<cell|a\<times\>b<rsup|\<um\>1><htab|5mm>(a,b\<in\>\<cal-E\>,b<rsup|\<um\>1><with|mode|text|
    defined>)>>|<row|<cell|a<rsup|0>>|<cell|=>|<cell|1<htab|5mm>(a\<in\>\<cal-E\>)>>|<row|<cell|a<rsup|1>>|<cell|=>|<cell|a<htab|5mm>(a\<in\>\<cal-E\>)>>|<row|<cell|a<rsup|b+c>>|<cell|=>|<cell|a<rsup|b>\<times\>a<rsup|c><htab|5mm>(a\<in\>\<cal-E\>,b\<in\>\<cal-E\>,c\<in\>\<cal-E\>)>>|<row|<cell|0<rsup|a>>|<cell|=>|<cell|0<htab|5mm>(a\<in\>\<cal-E\>,0<rsup|a><with|mode|text|
    defined>)>>|<row|<cell|1<rsup|a>>|<cell|=>|<cell|1<htab|5mm>(a\<in\>\<cal-E\>)>>>>
  </eqnarray*>

  It can be checked that the above representation for
  <with|mode|math|\<cal-E\>> is canonical for this set of axioms.

  <\warning>
    Bug to correct: we should not have <with|mode|math|(x\<times\>y)<rsup|a>/(x\<times\>y)=(x\<times\>y)<rsup|a-1>>,
    but only <with|mode|math|(x\<times\>y)<rsup|a>/(x\<times\>y)=(x\<times\>y)<rsup|a>\<times\>x<rsup|\<um\>1>\<times\>y<rsup|\<um\>1>>.
  </warning>

  Optionally, further constraints may be imposed on the elements of
  <with|mode|math|\<cal-E\>>, so as to obtain canonical representations for
  extensions of the above theory with the following optional axioms:

  <\eqnarray*>
    <tformat|<table|<row|<cell|a\<times\>(b+c)>|<cell|=>|<cell|a\<times\>b+a\<times\>c<htab|5mm>(a\<in\>\<cal-S\>,b\<in\>\<cal-E\>,c\<in\>\<cal-E\>,)>>|<row|<cell|a\<times\>(b+c)>|<cell|=>|<cell|a\<times\>b+a\<times\>c<htab|5mm>(a,b,c\<in\>\<cal-E\>)>>|<row|<cell|(a<rsup|b>)<rsup|c>>|<cell|=>|<cell|a<rsup|b\<times\>c><htab|5mm>(a\<in\>\<cal-E\>,b\<in\>\<cal-E\>,c\<in\>\<cal-E\>)>>|<row|<cell|(a\<times\>b)<rsup|c>>|<cell|=>|<cell|a<rsup|c>\<times\>b<rsup|c><htab|5mm>(a\<in\>\<cal-E\>,b\<in\>\<cal-E\>,c\<in\>\<cal-E\>)>>>>
  </eqnarray*>

  When allowing general distributivity, notice that we may need a rule to
  extract the ``integer part'' of scalars, so as to rewrite
  <with|mode|math|(a+b)<rsup|n+x>=<big|sum><rsub|i=0><rsup|n><binom|n|i>\<times\>a<rsup|i>\<times\>b<rsup|n-i>\<times\>(a+b)<rsup|x>>.
  Alternatively, one may disallow for general powers in the construction of
  power-products <with|mode|math|\<cal-P\>> (and only allow for integer
  powers or scalar powers, for instance). In that case, one should allow for
  more general powers in<nbsp><with|mode|math|\<cal-T\>>. Again
  alternatively, one may represent expressions in <with|mode|math|\<cal-E\>>
  as polynomials and only define a symbolic power operation on
  <with|mode|math|\<cal-E\>>.

  <tmdoc-copyright|2004|Joris van der Hoeven>

  <tmdoc-license|Permission is granted to copy, distribute and/or modify this
  document under the terms of the <hlink|GNU General Public
  License|http://www.gnu.org/licenses/gpl.txt>. If you don't have this file,
  write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.>
</body>