
/******************************************************************************
* MODULE     : string.hpp
* DESCRIPTION: Strings with possible zero-characters
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __STRING_HPP
#define __STRING_HPP
#include "base.hpp"
#ifdef MML_OLD_CPP
#include <strstream>
using std::ostrstream;
typedef ostrstream ostringstream;
#else
#include <sstream>
using std::ostringstream;
#endif
namespace mml {

class string;
class string_rep REP_STRUCT {
private:
  char* a;  // the string
  nat   n;  // length of string
  nat   l;  // allocated number of bytes

public:
  inline string_rep (nat n2): a(new char[n2]), n(n2), l(n2) {}
  inline ~string_rep () { delete[] a; }
  void extend (nat n);

  friend class string;
  friend inline nat N (const string& s);
  friend inline char* S (string& s);
  friend inline const char* S (const string& s);
  friend string copy (const string& s);
  friend string  operator *  (const string& s, const string& t);
};

class string {
INDIRECT_PROTO (string, string_rep)
public:
  inline string (nat n=0): rep (new string_rep (n)) {}
  string (char c);
  string (char *s);
  string (char *s, nat n);

  inline friend nat N (const string& s) { return s.rep->n; }
  inline friend char* S (string& s) { s.secure (); return s.rep->a; }
  inline friend const char* S (const string& s) { return s.rep->a; }
  inline char operator [] (nat i) const { return rep->a[i]; }
  inline char& operator () (nat i) { secure (); return rep->a[i]; }
  string operator () (nat start, nat end) const;
  string& operator << (char c);
  string& operator << (const string& t);
  bool operator == (char* s) const;
  bool operator != (char* s) const;
  bool operator == (const string& s) const;
  bool operator != (const string& s) const;
  friend nat hash (const string& s);
  friend string copy (const string& s);
  friend ostream& operator << (ostream& out, const string& s);
  friend string  operator *  (const string& s, const string& t);

  friend class string_rep;
};
INDIRECT_IMPL (string, string_rep)

bool    operator <  (const string& s, const string& t);
bool    operator <= (const string& s, const string& t);
bool    operator >  (const string& s, const string& t);
bool    operator >= (const string& s, const string& t);

/******************************************************************************
* Print arbitrary objects to strings
******************************************************************************/

template<class C> string
print_to_string (const C& data) {
  ostringstream out;
  out << data;
#ifdef MML_OLD_CPP
  if (out.pcount () == 0) return "";
  else return string (out.str(), out.pcount ());
#else
  std::string s= out.str();
  return string ((char*) ((void*) s.data()), s.length());
#endif
}

} // namespace mml
#endif // __STRING_HPP
