
/******************************************************************************
* MODULE     : interval_add.cpp
* DESCRIPTION: Addition of intervals
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
operator + (const interval& x, const interval& y) {
  if (is_error (x)) return x;
  if (is_error (y)) return y;
  if (has_low_prec (x)) {
    if (has_low_prec (y)) {
      interval_rep* r= raw_interval ();
      mpfr_add (r->min, x->min, y->min, GMP_RNDD);
      mpfr_add (r->max, x->max, y->max, GMP_RNDU);
      return r;
    }
    else {
      interval_rep* r= raw_interval (prec (y));
      lp_dyadic mid, rad;
      mpfr_add (*mid, x->min, x->max, GMP_RNDN);
      mpfr_mul_2si (*mid, *mid, -1, GMP_RNDN);
      mpfr_sub (*rad, x->max, x->min, GMP_RNDU);
      mpfr_mul_2si (*rad, *rad, -1, GMP_RNDU);
      mpfr_add (r->mid, *mid, y->mid, GMP_RNDN);
      CHECK_OVERFLOW (r);
      mpfr_set_ui_2si (r->rad, 1, mpfr_get_step (r->mid), GMP_RNDU);
      mpfr_add (r->rad, r->rad, y->rad, GMP_RNDU);
      mpfr_add (r->rad, r->rad, *rad, GMP_RNDU);
      return r->finish_high ();      
    }
  }
  else {
    if (has_low_prec (y)) {
      interval_rep* r= raw_interval (prec (x));
      lp_dyadic mid, rad;
      mpfr_add (*mid, y->min, y->max, GMP_RNDN);
      mpfr_mul_2si (*mid, *mid, -1, GMP_RNDN);
      mpfr_sub (*rad, y->max, y->min, GMP_RNDU);
      mpfr_mul_2si (*rad, *rad, -1, GMP_RNDU);
      mpfr_add (r->mid, *mid, x->mid, GMP_RNDN);
      CHECK_OVERFLOW (r);
      mpfr_set_ui_2si (r->rad, 1, mpfr_get_step (r->mid), GMP_RNDU);
      mpfr_add (r->rad, r->rad, x->rad, GMP_RNDU);
      mpfr_add (r->rad, r->rad, *rad, GMP_RNDU);
      return r->finish_high ();      
    }
    else {
      mp_exp_t expo_x= expo_mid (x);
      mp_exp_t expo_y= expo_mid (y);
      mp_exp_t step_x= expo_x - prec (x);
      mp_exp_t step_y= expo_y - prec (y);
      mp_exp_t expo_r= max (expo_x, expo_y) + 1;
      mp_exp_t step_r= max (step_x, step_y);
      interval_rep* r= raw_interval (expo_r - step_r);
      mpfr_add (r->mid, x->mid, y->mid, GMP_RNDN);
      CHECK_OVERFLOW (r);
      mpfr_set_ui_2si (r->rad, 1, step_r, GMP_RNDU);
      mpfr_add (r->rad, r->rad, x->rad, GMP_RNDU);
      mpfr_add (r->rad, r->rad, y->rad, GMP_RNDU);
      return r->finish_high ();
    }
  }
}

} // namespace mml
