
/******************************************************************************
* MODULE     : context_base.hpp
* DESCRIPTION: Common utilities for contexts for symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __CONTEXT_BASE_HPP
#define __CONTEXT_BASE_HPP
#include "context.hpp"
#include "function.hpp"
namespace mml {

  //#define evaluator function<symbolic,symbolic>
  //#define evaluator_rep function_rep<symbolic,symbolic>
typedef function<symbolic,symbolic> evaluator;
typedef function_rep<symbolic,symbolic> evaluator_rep;
typedef function<bool,symbolic> selector;

class context_base_rep: public context_rep {
public:
  context                   base;
  table<context,context>    lcc;
  table<evaluator,symbolic> dispatch;

public:
  context_base_rep (const context& base, const symbolic& id);
  virtual bool extends (const context& c) const;
  virtual context least_common_context (const context& c);
  virtual context fetch_lcc (const context& c);
  virtual symbolic eval (const symbolic& s) const;
  virtual symbolic eval_atomic (const symbolic& s) const;
  virtual symbolic eval_compound (const symbolic& s) const;

  virtual symbolic add (const symbolic& s1, const symbolic& s2) const;
  virtual symbolic sub (const symbolic& s1, const symbolic& s2) const;
  virtual symbolic neg (const symbolic& s1) const;
  virtual symbolic mul (const symbolic& s1, const symbolic& s2) const;
  virtual symbolic div (const symbolic& s1, const symbolic& s2) const;
  virtual symbolic pow (const symbolic& s1, const symbolic& s2) const;
};

context basic_context (const context& base); // base must extend scalars
context commutative_context (const context& base, const selector& sel);

} // namespace mml
#endif // __CONTEXT_BASE_HPP
