
/******************************************************************************
* MODULE     : symbolic_defs.hpp
* DESCRIPTION: Global definitions for symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYMBOLIC_CST_HPP
#define __SYMBOLIC_CST_HPP
namespace mml {

enum symbolic_type {
  SYMBOLIC_UNKNOWN,
  SYMBOLIC_ATOM,
  SYMBOLIC_INTEGER,
  SYMBOLIC_RATIONAL,
  SYMBOLIC_COMPOUND,
  SYMBOLIC_SUM,
  SYMBOLIC_PRODUCT
};

enum add_type {
  ADD_INTEGER,
  ADD_RATIONAL,
  ADD_COMMUTATIVE,
  ADD_GENERIC
};

enum mul_type {
  MUL_INTEGER,
  MUL_RATIONAL,
  MUL_COMMUTATIVE,
  MUL_GENERIC
};

enum pow_type {
  POW_EXPO_LINEAR,
  POW_GENERIC
};

class symbolic;
extern symbolic SYMB_ZERO;
extern symbolic SYMB_ONE;
extern symbolic SYMB_LESSLESS;
extern symbolic SYMB_GTRGTR;
extern symbolic SYMB_IMPLIES;
extern symbolic SYMB_EQUIV;
extern symbolic SYMB_OR;
extern symbolic SYMB_XOR;
extern symbolic SYMB_AND;
extern symbolic SYMB_EQUAL;
extern symbolic SYMB_UNEQUAL;
extern symbolic SYMB_LESS;
extern symbolic SYMB_LESSEQ;
extern symbolic SYMB_GTR;
extern symbolic SYMB_GTREQ;
extern symbolic SYMB_FUNCTION;
extern symbolic SYMB_PLUS;
extern symbolic SYMB_TIMES;
extern symbolic SYMB_OVER;
extern symbolic SYMB_COMPOSE;
extern symbolic SYMB_NOT;
extern symbolic SYMB_MINUS;
extern symbolic SYMB_POWER;
extern symbolic SYMB_FACTORIAL;
extern symbolic SYMB_PRIME;
extern symbolic SYMB_DOT;
extern symbolic SYMB_APPLY;
extern symbolic SYMB_ACCESS;
extern symbolic SYMB_TUPLE;
extern symbolic SYMB_VTUPLE;
extern symbolic SYMB_SQTUPLE;

extern symbolic CONTEXT_FREE;
extern symbolic CONTEXT_BASIC;
extern symbolic CONTEXT_COMMUTATIVE;

struct arithmetic_props {
  unsigned add_neutral            : 1;
  unsigned add_opposite           : 1;
  unsigned add_associative        : 1;
  unsigned add_commutative        : 1;
  unsigned add_scalar             : 1;
  unsigned mul_zero               : 1;
  unsigned mul_neutral            : 1;
  unsigned mul_opposite           : 1;
  unsigned mul_associative        : 1;
  unsigned mul_commutative        : 1;
  unsigned mul_left_distributive  : 1;
  unsigned mul_right_distributive : 1;
  unsigned mul_scalar             : 1;
  unsigned pow_right_distributive : 1;
};

} // namespace mml
#endif // __extern symbolic SYMBOLIC_CST_HPP
