
/******************************************************************************
* MODULE     : context.cpp
* DESCRIPTION: Contexts for symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "context.hpp"
namespace mml {

/******************************************************************************
* Main methods of the context_rep class
******************************************************************************/

bool
context_rep::extends (const context& c) const {
  return c == me();
}

context
context_rep::least_common_context (const context& c) {
  return c;
}

symbolic
context_rep::eval (const symbolic& s) const {
  if (is_atomic (s)) return eval_atomic (L(s));
  else return eval_compound (s);
}

symbolic
context_rep::eval_atomic (const symbolic& s) const {
  return L(s);
}

symbolic
context_rep::eval_compound (const symbolic& s) const {
  if (s->type () != SYMBOLIC_COMPOUND) {
    nat i, n= N(s);
    vector<symbolic> v(n);
    for (i=0; i<n; i++) v(i)= s[i];
    return symbolic_compound (v);
  }
  else return s;
}

symbolic
context_rep::eval (const symbolic& s, bool recursive) const {
  if (is_compound (s) && recursive) {
    nat i, n= N(s);
    vector<symbolic> v(n);
    for (i=0; i<n; i++) v(i)= eval (s[i], true);
    return eval (symbolic_compound (v));
  }
  else return eval (s);
}

/******************************************************************************
* Fast treatment of common operations
******************************************************************************/

symbolic
context_rep::add (const symbolic& s1, const symbolic& s2) const {
  return eval (symbolic_compound (SYMB_PLUS, s1, s2));
}

symbolic
context_rep::sub (const symbolic& s1, const symbolic& s2) const {
  return eval (symbolic_compound (SYMB_MINUS, s1, s2));
}

symbolic
context_rep::neg (const symbolic& s1) const {
  return eval (symbolic_compound (SYMB_MINUS, s1));
}

symbolic
context_rep::mul (const symbolic& s1, const symbolic& s2) const {
  return eval (symbolic_compound (SYMB_TIMES, s1, s2));
}

symbolic
context_rep::div (const symbolic& s1, const symbolic& s2) const {
  return eval (symbolic_compound (SYMB_OVER, s1, s2));
}

symbolic
context_rep::pow (const symbolic& s1, const symbolic& s2) const {
  return eval (symbolic_compound (SYMB_POWER, s1, s2));
}

} // namespace mml
