
/******************************************************************************
* MODULE     : context_scalar.cpp
* DESCRIPTION: Context for common scalar types (integers and rationals)
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "context.hpp"
#include "symbolic_impl.hpp"
namespace mml {

class context_scalar_rep: public context_rep {
  context base;
public:
  inline context_scalar_rep (const context& base2):
    context_rep (symbolic_compound ("scalar", base2->id)), base (base2) {}
  bool extends (const context& c) const {
    return c == me () || base->extends (c); }
  context least_common_context (const context& c) {
    if (extends (c)) return me ();
    assert (c->extends (me ()));
    return c;
  }

  symbolic add (const symbolic& s1, const symbolic& s2) const {
    if ((s1->type() == SYMBOLIC_INTEGER || s1->type() == SYMBOLIC_RATIONAL) &&
	(s2->type() == SYMBOLIC_INTEGER || s2->type() == SYMBOLIC_RATIONAL))
      return rational_add (s1, s2);
    else return base->add (s1, s2);
  }
  symbolic sub (const symbolic& s1, const symbolic& s2) const {
    if ((s1->type() == SYMBOLIC_INTEGER || s1->type() == SYMBOLIC_RATIONAL) &&
	(s2->type() == SYMBOLIC_INTEGER || s2->type() == SYMBOLIC_RATIONAL))
      return rational_sub (s1, s2);
    else return base->sub (s1, s2);
  }
  symbolic neg (const symbolic& s1) const {
    if (s1->type() == SYMBOLIC_INTEGER || s1->type() == SYMBOLIC_RATIONAL)
      return rational_neg (s1);
    else return base->neg (s1);
  }
  symbolic mul (const symbolic& s1, const symbolic& s2) const {
    if ((s1->type() == SYMBOLIC_INTEGER || s1->type() == SYMBOLIC_RATIONAL) &&
	(s2->type() == SYMBOLIC_INTEGER || s2->type() == SYMBOLIC_RATIONAL))
      return rational_mul (s1, s2);
    else return base->mul (s1, s2);
  }
  symbolic div (const symbolic& s1, const symbolic& s2) const {
    if ((s1->type() == SYMBOLIC_INTEGER || s1->type() == SYMBOLIC_RATIONAL) &&
	(s2->type() == SYMBOLIC_INTEGER || s2->type() == SYMBOLIC_RATIONAL))
      return rational_div (s1, s2);
    else return base->div (s1, s2);
  }
};

context
scalar_context (const context& c) {
  return new context_scalar_rep (c);
}

} // namespace mml
