
/******************************************************************************
* MODULE     : symbolic_atom.cpp
* DESCRIPTION: Atomic symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic_aux.hpp"
namespace mml {

class symbolic_atom_rep: public symbolic_rep {
  string data;

protected:
  // accessors and syntactic identity
  symbolic_type type () const { return SYMBOLIC_ATOM; }
  nat hash () const { return mml::hash (data); }
  bool identical (symbolic_rep* rep) const {
    if (rep->type() != SYMBOLIC_ATOM) return false;
    return data == ((symbolic_atom_rep*) rep) -> data; }
  symbolic flatten (bool recursive) const {
    (void) recursive;
    return symbolic (data); }

  bool is_atomic () { return true; }
  string leaf () { return data; }
  nat arity () { return 0; }
  symbolic child (nat i) {
    assert (false); // not a compound object
    return symbolic (); }

public:
  symbolic_atom_rep (const string& data2): data (data2) {}
};

/******************************************************************************
* Constructors
******************************************************************************/

symbolic::symbolic (char* s) {
  rep= memorize (new symbolic_atom_rep (string (s)));
}

symbolic::symbolic (const string& s) {
  rep= memorize (new symbolic_atom_rep (s));
}

} // namespace mml
