
/******************************************************************************
* MODULE     : symbolic_compound.cpp
* DESCRIPTION: Compound symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic.hpp"
namespace mml {

class symbolic_compound_rep: public symbolic_rep {
public:
  vector<symbolic> data;

protected:
  // accessors and syntactic identity
  symbolic_type type () const { return SYMBOLIC_COMPOUND; }
  nat hash () const { return mml::hash (data); }
  bool identical (symbolic_rep* rep) const {
    if (rep->type() != SYMBOLIC_COMPOUND) return false;
    return data == ((symbolic_compound_rep*) rep)->data;
  }
  symbolic flatten (bool recursive) const {
    nat i, n= dim (data);
    vector<symbolic> v= vector<symbolic> (symbolic (), n);
    for (i=0; i<n; i++)
      v(i)= mml::flatten (data[i], recursive);
    return symbolic_compound (v); }

  bool is_atomic () { return false; }
  string leaf () {
    assert (false); // not an atomic object
    return ""; }
  nat arity () { return dim (data) - 1; }
  symbolic child (nat i) { return data[i]; }

public:
  symbolic_compound_rep (const vector<symbolic>& data2): data (data2) {
    assert (dim (data) > 0); }
};

/******************************************************************************
* Constructors
******************************************************************************/

symbolic
symbolic_compound (const vector<symbolic>& v) {
  return new symbolic_compound_rep (v);
}

symbolic
symbolic_compound (const symbolic& fun, const vector<symbolic>& v) {
  return new symbolic_compound_rep (append (vec (fun), v));
}

symbolic
symbolic_compound (const symbolic& fun) {
  return new symbolic_compound_rep (vec (fun));
}

symbolic
symbolic_compound (const symbolic& fun, const symbolic& s1) {
  return new symbolic_compound_rep (vec (fun, s1));
}

symbolic
symbolic_compound (const symbolic& fun,
		   const symbolic& s1, const symbolic& s2) {
  return new symbolic_compound_rep (vec (fun, s1, s2));
}

} // namespace mml
