
/******************************************************************************
* MODULE     : symbolic_integer.cpp
* DESCRIPTION: Symbolic integers
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic_aux.hpp"
#include "rational.hpp"
namespace mml {

class symbolic_integer_rep: public symbolic_rep {
public:
  integer data;

protected:
  // accessors and syntactic identity
  symbolic_type type () const { return SYMBOLIC_INTEGER; }
  nat hash () const { return mml::hash (data); }
  bool identical (symbolic_rep* rep) const {
    if (rep->type() != SYMBOLIC_INTEGER) return false;
    return data == ((symbolic_integer_rep*) rep)->data; }
  symbolic flatten (bool recursive) const {
    (void) recursive;
    return print_to_string<integer> (data); }

  // arithmetic
  add_type get_add_type () const { return ADD_INTEGER; }
  mul_type get_mul_type () const { return MUL_INTEGER; }
  pow_type get_pow_type () const { return POW_EXPO_LINEAR; }

public:
  symbolic_integer_rep (const integer& data2): data (data2) {}
};

/******************************************************************************
* Constructors and conversions
******************************************************************************/

symbolic::symbolic () {
  rep= SYMB_ZERO.rep; INC_COUNT (rep);
}

symbolic::symbolic (int i) {
  rep= memorize (new symbolic_integer_rep (integer (i)));
}

symbolic::symbolic (const integer& i) {
  rep= memorize (new symbolic_integer_rep (i));
}

symbolic_rep*
symbolic_integer (const integer& i) {
  return new symbolic_integer_rep (i);
}

integer
as_integer (const symbolic& s) {
  assert (s->type () == SYMBOLIC_INTEGER);
  return ((const symbolic_integer_rep*) access(s)) -> data;
}

/******************************************************************************
* Arithmetic
******************************************************************************/

symbolic
integer_add (const symbolic& s1, const symbolic& s2) {
  return as_integer (s1) + as_integer (s2);
}

symbolic
integer_sub (const symbolic& s1, const symbolic& s2) {
  return as_integer (s1) - as_integer (s2);
}

symbolic
integer_neg (const symbolic& s1) {
  return -as_integer (s1);
}

symbolic
integer_mul (const symbolic& s1, const symbolic& s2) {
  return as_integer (s1) * as_integer (s2);
}

symbolic
integer_div (const symbolic& s1, const symbolic& s2) {
  return as_rational (s1) / as_rational (s2);
}

} // namespace mml
