
/******************************************************************************
* MODULE     : symbolic_sum.cpp
* DESCRIPTION: Commutative sums of a finite number of symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic_impl.hpp"
namespace mml {

class symbolic_sum_rep: public symbolic_rep {
public:
  symbolic_commutative sum;

protected:
  // accessors and syntactic identity
  symbolic_type type () const { return SYMBOLIC_SUM; }
  nat hash () const { return 147258369 ^ mml::hash (sum); }
  bool identical (symbolic_rep* rep) const {
    if (rep->type() != SYMBOLIC_SUM) return false;
    return sum == ((symbolic_sum_rep*) rep)->sum;
  }
  symbolic flatten (bool recursive) const {
    iterator<symbolic> it= entries (sum);
    symbolic acc= 0;
    while (busy (it)) {
      symbolic term;
      symbolic mon  = *it;
      symbolic coeff= sum[mon];
      if (coeff == 1) term= mon;
      else if (mon == 1) term= coeff;
      else term= symbolic_compound ("*", coeff, mon);
      if (acc == 0) acc= term;
      else acc= symbolic_compound ("+", term, acc);
      ++it;
    }
    return mml::flatten (acc, recursive);
  }

  // arithmetic
  add_type get_add_type () const { return ADD_COMMUTATIVE; }
  //mul_type get_mul_type () const { return ?; }
  //pow_type get_pow_type () const { return ?; }

public:
  symbolic_sum_rep (const symbolic_commutative& sum2): sum (sum2) {}
};

/******************************************************************************
* Constructors
******************************************************************************/

symbolic
symbolic_sum (const symbolic_commutative& sum) {
  // cout << "sum " << sum << "\n";
  switch (N(sum)) {
  case 0:
    return SYMB_ZERO;
  case 1:
    {
      symbolic f= *entries (sum);
      if (f == SYMB_ONE) return sum[f];
      if (sum[f] == SYMB_ONE) return f;
      return new symbolic_sum_rep (sum);
    }
  default:
    return new symbolic_sum_rep (sum);
  }
}

symbolic_commutative
as_symbolic_sum (const symbolic& s) {
  if (s->type () == SYMBOLIC_SUM)
    return ((symbolic_sum_rep*) access (s)) -> sum;
  else if (is_scalar (s)) {
    symbolic_commutative sum (SYMB_ZERO);
    if (s != SYMB_ZERO) sum (SYMB_ONE)= s;
    return sum;
  }
  else {
    symbolic_commutative sum (SYMB_ZERO);
    sum (s)= SYMB_ONE;
    return sum;
  }
}

/******************************************************************************
* Operations
******************************************************************************/

symbolic
sum_add (const symbolic& s1, const symbolic& s2) {
  return symbolic_sum (as_symbolic_sum (s1) + as_symbolic_sum (s2));
}

symbolic
sum_sub (const symbolic& s1, const symbolic& s2) {
  return symbolic_sum (as_symbolic_sum (s1) - as_symbolic_sum (s2));
}

symbolic
sum_neg (const symbolic& s1) {
  return symbolic_sum (-as_symbolic_sum (s1));
}

symbolic
sum_scalar_mul (const symbolic& sc, const symbolic& s1) {
  if (sc == SYMB_ZERO) return SYMB_ZERO;
  else if (sc == SYMB_ONE) return s1;
  else {
    // return symbolic_sum (sc * as_symbolic_sum (s1));
    symbolic_commutative sum= as_symbolic_sum (s1);
    if (N (sum) <= 1) return symbolic_sum (sc * as_symbolic_sum (s1));
    else {
      symbolic_commutative new_sum (SYMB_ZERO);
      new_sum (s1)= sc;
      return new symbolic_sum_rep (new_sum);
    }
  }
}

} // namespace mml
